import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Window
import QtQuick.VirtualKeyboard.Settings

import Core
import Dialog
import Sco

import "widgets"

/**
 * @brief Основное окно
 */
ApplicationWindow {
    id: root
    objectName: "qmlMainWindow"
    width: Screen.desktopAvailableWidth
    height: Screen.desktopAvailableHeight
    visible: true

    function topPanelColor(c: color) {
        // Один из способов проверить что цвет валидный https://bugreports.qt.io/browse/QTBUG-78325
        return Qt.colorEqual(Qt.rgba(c.r, c.g, c.b, c.a), c) ? c : Style.color("background")
    }

    ColumnLayout {
        anchors.fill: parent
        spacing: 0

        TopPanel {
            id: topPanel
            z: 1
            visible: !workArea.currentItem || !workArea.currentItem.Context.fullScreen || dialogWindow.visible

            property color baseColor
            property color targetColor
            property real targetOpacity: workArea.currentItem ? workArea.currentItem.opacity : 1
            backgroundColor: Qt.tint(baseColor, Qt.rgba(targetColor.r, targetColor.g, targetColor.b, targetOpacity))

            onLogo: () => mainWindow.logo()

            Layout.fillWidth: true
        }

        StackView {
            id: workArea

            pushEnter: null
            pushExit: null
            popEnter: null
            popExit: null
            replaceEnter: null
            replaceExit: null

            Layout.fillWidth: true
            Layout.fillHeight: true

            function removeItem(item: Item) {
                if (this.get(item.StackView.index) !== item)
                    return

                this.pushEnter = null
                this.pushExit = null
                this.popEnter = null
                this.popExit = null

                const tmp = []
                while (tmp[tmp.length - 1] !== item && workArea.depth > 1) {
                    const popped = this.pop(StackView.Immediate)
                    tmp.push(popped)
                }

                if (tmp[tmp.length - 1] === item)
                    tmp.pop()

                if (tmp.length > 0) {
                    this.popEnter = tmp[tmp.length - 1].popEnter
                    this.push(tmp, StackView.PopTransition)
                } else if (this.currentItem) {
                    this.popEnter = workArea.currentItem.popEnter
                    this.replace(workArea.currentItem, workArea.currentItem, StackView.PopTransition)
                }
            }
        }
    }

    Dialog {
        id: modalBackground
        anchors.centerIn: Overlay.overlay
        closePolicy: Popup.NoAutoClose
        modal: true
        z: 3; width: 0; height: 0

        Overlay.modal: Rectangle {
            color: Style.color("modalBackground")
        }

        Overlay.onReleased: {
            keyboardWindow.hideKeyboard()
        }

        // Дублируем лого, так как на оригинальный объект нажать теперь нельзя
        Item {
            parent: modalBackground.contentItem
            z: 5
            property var pos: parent ? parent.mapFromItem(Overlay.overlay, topPanel.leftPadding, topPanel.topPadding) : Qt.point(0, 0)
            x: pos.x; y: pos.y
            height: topPanel.appLogo.height
            width: topPanel.appLogo.width

            TapHandler {
                gesturePolicy: TapHandler.WithinBounds
                onTapped: topPanel.logo()
            }
        }
    }

    Window {
        id: dialogWindow
        flags: Qt.FramelessWindowHint
        color: "transparent"

        transientParent: root

        property string lastGeometry

        function updateGeometry() {
            x = root.x + (root.width - dialogBox.implicitWidth) / 2
            y = root.y + (root.height - dialogBox.implicitHeight) / 2
            width = dialogBox.implicitWidth
            height = dialogBox.implicitHeight
            const geometry = width + "x" + height
            if (geometry !== lastGeometry) {
                lastGeometry = geometry
                Utils.applyWindowRadius(this, dialogBox.Style.border.radius)
            }
        }

        onActiveChanged: () => {
            if (dialogWindow.visible && Utils.focusWidget() === null)
                dialogWindow.requestActivate()
        }

        SGroupBox {
            id: dialogBox
            Style.class: "modal"
            anchors.fill: parent
            padding: 0
            clip: true

            onImplicitHeightChanged: () => dialogWindow.updateGeometry()

            MouseArea {
                x: 0; y: 0; z: 1
                width: 100; height: 100

                onClicked: () => topPanel.logo()
            }
        }
    }

    Window {
        id: keyboardWindow
        flags: Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint | Qt.Popup | Qt.NoFocus
        x: root.x + (root.width - width) / 2

        width: Style.parameter("geometry.keyboardWindow.width", 1010)
        height: Style.parameter("geometry.keyboardWindow.height", 720)

        property real radius: keyboardContainer.Style.border.radius
        property real popupIndent: Style.parameter("geometry.keyboardWindow.popupIndent", 35)

        Component.onCompleted: () => Utils.applyWindowRadius(this, radius)
        onRadiusChanged: () => Utils.applyWindowRadius(this, radius)

        // При скруглении окна остаются небольшие артефакты по краям.
        // Перекрашиваем окно, чтобы скрыть их.
        property real borderWidth: keyboardContainer.Style.border.width
        property color borderColor: keyboardContainer.Style.border.color
        property color backgroundColor: keyboardContainer.Style.background.color
        color: borderWidth > 0 ? borderColor : backgroundColor

        function updateFocusedItem() {
            let newFocusedItem
            if (Utils.isQmlWindow()) {
                newFocusedItem = dialogWindow.visible ? dialogWindow.activeFocusItem : root.activeFocusItem
            } else {
                newFocusedItem = Utils.focusWidget()
                if (focusedItem === void 0 || focusedItem !== void 0 && focusedItem.showKeyboard !== true)
                    return
            }
            if (focusedItem !== newFocusedItem)
                resetFocusedItem()
            if (focusedItem !== void 0 && focusedItem.text !== void 0) {
                if (typeof focusedItem.displayTextChanged === 'function') {
                    focusedItemConnectionDisplayTextChanged = function() { keyboard.text = focusedItem.displayText }
                    focusedItem.displayTextChanged.connect(focusedItemConnectionDisplayTextChanged)
                    keyboard.text = focusedItem.displayText
                }
                if (typeof focusedItem.focusChanged === 'function') {
                    focusedItemConnectionDisplayTextChanged = function() { updateFocusedItem() }
                    focusedItem.focusChanged.connect(focusedItemConnectionFocusChanged)
                }
                if (typeof focusedItem.selectText === 'function')
                    focusedItem.selectText()
            } else {
                keyboard.text = ""
            }

            showKeyboard()
        }

        function resetFocusedItem() {
            if (focusedItem === void 0)
                return

            if (focusedItemConnectionDisplayTextChanged !== void 0)
                focusedItem.displayTextChanged.disconnect(focusedItemConnectionDisplayTextChanged)
            if (focusedItemConnectionFocusChanged !== void 0)
                focusedItem.focusChanged.disconnect(focusedItemConnectionFocusChanged)
            if (typeof focusedItem.clearFocus === 'function')
                focusedItem.clearFocus()
            focusedItem = void 0
            focusedItemConnectionDisplayTextChanged = void 0
            focusedItemConnectionFocusChanged = void 0
        }

        function showKeyboard() {
            keyboardContainer.state = "show"
            keyboardWindow.show()
            keyboardWindow.raise()
            Qt.callLater(() => keyboard.resetLocale())
        }

        function hideKeyboard() {
            resetFocusedItem()
            keyboardContainer.state = "hide"
            Qt.callLater(Qt.inputMethod.hide)
        }

        function confirm() {
            Qt.inputMethod.commit()
            hideKeyboard()
        }

        function connectFloatKeyboard(item) {
            let locale = item.floatKeyboardDefaultLocale
            let type = item.floatKeyboardLayoutType
            item.onFloatKeyboardLayoutTypeChanged.connect(() =>
                resetKeyboardLayout(item.floatKeyboardDefaultLocale, item.floatKeyboardLayoutType)
            )
            resetKeyboardLayout(locale, type)

            if (typeof item.hideKeyboard === 'function')
                item.hideKeyboard.connect(() => hideKeyboard())
        }

        function resetKeyboardLayout(locale, type) {
            keyboard.defaultLocale = locale.length > 0 ? locale : "ru_RU"
            keyboard.layoutType = type.length > 0 ? type : "main"
            keyboard.resetLocale()
        }

        property bool enabled: allowFloatKeyboard && Qt.inputMethod.visible
        property var focusedItem
        property var focusedItemConnectionDisplayTextChanged
        property var focusedItemConnectionFocusChanged
        property bool allowFloatKeyboard: false
        onEnabledChanged: enabled ? updateFocusedItem() : resetFocusedItem()

        SGroupBox {
            id: keyboardContainer
            anchors.fill: parent
            Style.class: "fullSizedKeyboardWindow"
            FullSizeKeyboard {
                id: keyboard

                anchors.fill: parent
                onConfirm: keyboardWindow.confirm()
            }
            state: "hide"
            states: [
                State { name: "hide"; PropertyChanges { target: keyboardWindow; y: root.height; explicit: true } },
                State { name: "show"; PropertyChanges {
                        target: keyboardWindow; y: root.height - height - keyboardWindow.popupIndent
                    }
                }
            ]
            transitions: [
                Transition { from: "hide"; to: "show"
                    SmoothedAnimation {
                        velocity: Style.animationVelocity("transition.keyboard", 2000)
                        target: keyboardWindow;
                        easing.type: Easing.OutCirc
                        property: "y"
                    }
                },
                Transition { from: "show"; to: "hide"
                    SequentialAnimation {
                        SmoothedAnimation {
                            velocity: Style.animationVelocity("transition.keyboard", 3500)
                            target: keyboardWindow;
                            easing.type: Easing.InCirc
                            property: "y"
                        }
                        ScriptAction { script: keyboardWindow.hide() }
                    }
                }
            ]
        }
    }

    Component.onCompleted: () => {
        let langs = Config.getList("I18n:keyboardLayouts", ["ru", "en"])
        let locales = []
        for (let i = 0; i < langs.length; i++) {
            switch(langs[i]) {
                case "ru": locales.push("ru_RU"); break;
                case "en": locales.push("en_US"); break;
            }
        }
        VirtualKeyboardSettings.activeLocales = locales;
        requestActivate()
    }

    Connections {
        target: mainWindow

        function onShowMainContext(context, item) {
            if (item) {
                keyboardWindow.connectFloatKeyboard(item)
                if (!Sco.startup) {
                    workArea.pushEnter = item.pushEnter
                    workArea.pushExit = null
                    workArea.popEnter = item.popEnter
                    workArea.popExit = null
                }

                if (workArea.get(item.StackView.index) !== item) {
                    if (workArea.currentItem) {
                        workArea.pushExit = workArea.currentItem.pushExit
                        topPanel.baseColor = topPanelColor(workArea.currentItem.Context.topPanelColor)
                    }

                    workArea.push(item)
                    context.removed.connect(() => workArea.removeItem(item))
                } else {
                    workArea.pop(item)
                }

                topPanel.targetColor = topPanelColor(item.Context.topPanelColor)

                root.show()
                root.raise()
                item.focus = true
            } else {
                root.hide()
            }

            modalBackground.visible = false
            keyboardWindow.hideKeyboard()
        }

        function onShowModalContext(context, item, emitOnModalLoaded) {
            if (dialogBox.contentItem) {
                if (dialogWindow.visible)
                    dialogWindow.hide()
                dialogBox.contentItem = null
            }

            if (item) {
                keyboardWindow.allowFloatKeyboard = item.allowFloatKeyboard
                keyboardWindow.connectFloatKeyboard(item)

                dialogBox.contentItem = item
                dialogWindow.updateGeometry()
                dialogWindow.requestActivate()

                // если необходимо, скрываем форму
                if (item.hide) {
                    if (modalBackground.visible === true)
                        modalBackground.visible = false
                    item.visible = false
                    return
                }

                item.visible = true

                // отображение формы с задержкой
                delayTimer.stop()
                delayTimer.interval = item.showDelay ?? 0
                delayTimer.triggered.connect(() => {
                    if (dialogBox.contentItem !== item)
                        return

                    if (emitOnModalLoaded)
                        mainWindow.modalLoaded()
                    modalBackground.visible = true
                    dialogWindow.show()

                    // костыль: два callLater, иначе диалог рисуется раньше заднего фона
                    Qt.callLater(() =>  {
                        Qt.callLater(() => {
                            dialogWindow.raise()
                            item.focus = true
                            if (typeof item.activated === 'function')
                                item.activated()
                            // чтобы новая qml-форма не загораживала собой окна с заглушками
                            PluginManager.internalAsync("HWDUMMY_RELOADHWDUMMYFORMS")
                        })
                    })
                })
                delayTimer.restart()
            } else if (root.visible && workArea.currentItem) {
                keyboardWindow.allowFloatKeyboard = workArea.currentItem.allowFloatKeyboard
                if (typeof workArea.currentItem.activated === 'function')
                    workArea.currentItem.activated()
            } else {
                keyboardWindow.allowFloatKeyboard = true
            }
        }

        function onModalChanged(modal) {
            // отображение заднего фона с задержкой
            if (DialogState.newProgressState === null) {
                modalBackground.visible = modal
                return
            }

            if (DialogState.newProgressState.hide)
                return
            const delay = DialogState.newProgressState.delay
            if (!(delay && delay > 0) || !modal)
                modalBackground.visible = modal
        }
    }

    Timer {
        id: delayTimer
        repeat: false
        interval: 0
    }
}
