import QtQml.Base
import QtQuick
import QtQuick.Layouts

import Core
import I18n
import Dialog

import "../core"
import "../widgets"

BaseForm {
    id: root
    Style.class: "customerAddressForm"

    allowFloatKeyboard: true
    floatKeyboardLayoutType: isEmail ? "email" : "dialpad"
    floatKeyboardDefaultLocale: "en_US"

    signal hideKeyboard()
    property bool isEmail: selectedType == DialogTypes.CustomerAddressType.Email
    property var action: DialogState.actions[Context.id.value]
    property int selectedType: action.types[0]

    Connections {
        target: DialogState

        function onInputCustomerAddress(address, type) {
            root.selectedType = type
            inputField.text = address
        }
    }

    onSelectedTypeChanged: () => inputField.text = ""

    onActivated: () => {
        if (action.types.length <= 1)
            inputField.forceActiveFocus()
    }

    Context.inputSources: InputSources.Scanner

    function accept() {
        if (!inputField.acceptableInput)
            return

        action.address = inputField.text
        action.type = selectedType
        removeContext()
    }

    function reject() {
        action.address = ""
        removeContext()
    }

    contentItem: ColumnLayout {
        objectName: "mainLayout"
        spacing: 40

        SGroupBox {
            objectName: "mainGroup"
            Style.class: "customerAddressMain"
            leftPadding: 173
            rightPadding: 173
            topPadding: 280

            Layout.fillWidth: true
            Layout.fillHeight: true

            onClicked: () => {
                this.forceActiveFocus(Qt.MouseFocusReason)
                root.hideKeyboard()
            }

            contentItem: ColumnLayout {
                objectName: "contentLayout"
                spacing: 60

                SLabel {
                    objectName: "titleLabel"
                    Style.class: "customerAddressTitle"
                    horizontalAlignment: SLabel.AlignHCenter
                    wrapMode: SLabel.WordWrap
                    Tr { code: "addressTitle" }
                    Layout.fillWidth: true
                }

                ColumnLayout {
                    objectName: "typeLayout"
                    spacing: 40
                    visible: action.types.length > 1

                    SLabel {
                        objectName: "typeTitleLabel"
                        Style.class: "customerAddressSelectType"
                        horizontalAlignment: SLabel.AlignHCenter
                        wrapMode: SLabel.WordWrap
                        Tr { fallback: "addressSelectType" }
                        Layout.fillWidth: true
                    }

                    RowLayout {
                        objectName: "typeButtonsLayout"
                        spacing: 40
                        Layout.alignment: Qt.AlignHCenter

                        ActionButton {
                            objectName: "typeEmailButton"
                            Style.class: "customerAddressType"
                            visible: root.action.types.find((type) => type == DialogTypes.CustomerAddressType.Email) != null
                            enabled: selectedType != DialogTypes.CustomerAddressType.Email

                            onClicked: () => root.selectedType = DialogTypes.CustomerAddressType.Email

                            Tr { code: "addressEmail" }

                            Layout.preferredWidth: 306.5
                            Layout.preferredHeight: 100
                        }

                        ActionButton {
                            objectName: "typeSmsButton"
                            Style.class: "customerAddressType"
                            visible: root.action.types.find((type) => type == DialogTypes.CustomerAddressType.Sms) != null
                            enabled: selectedType != DialogTypes.CustomerAddressType.Sms

                            onClicked: () => root.selectedType = DialogTypes.CustomerAddressType.Sms

                            Tr { code: "addressSms" }

                            Layout.preferredWidth: 306.5
                            Layout.preferredHeight: 100
                        }
                    }
                }

                SLabel {
                    objectName: "hintLabel"
                    Style.class: "customerAddressHint"
                    horizontalAlignment: SLabel.AlignHCenter
                    wrapMode: SLabel.WordWrap
                    Layout.fillWidth: true

                    Tr {
                        code: {
                            switch (selectedType) {
                            case DialogTypes.CustomerAddressType.Email: return "addressEmailMsg";
                            case DialogTypes.CustomerAddressType.Sms: return "addressSmsMsg";
                            default: return "";
                            }
                        }
                    }
                }

                SGroupBox {
                    objectName: "inputFieldGroup"
                    Style.class: "customerAddressInputField"
                    Layout.fillWidth: true

                    contentItem: STextEdit {
                        objectName: "inputField"
                        id: inputField
                        anchors.fill: parent
                        clip: true
                        leftPadding: 15
                        rightPadding: 15
                        horizontalAlignment: Qt.AlignHCenter
                        onCursorPositionChanged: {
                            if (selectedType == DialogTypes.CustomerAddressType.Sms)
                                fixCursorPosition()
                        }

                        // Костыль, потому что inputMask, неожиданно, является частью отображаемого текста.
                        // Так мы гарантируем, что позиция курсора будет находиться на нужной нам позиции
                        function fixCursorPosition() {
                            let index = this.text.replace(/\(|\)|-/g, "").length
                            if (index < 5)
                                index += 1
                            else if (index < 8)
                                index += 2
                            else if (index < 10)
                                index += 3
                            else
                                index += 4
                            cursorPosition = index
                        }

                        inputMethodHints: {
                            switch (selectedType) {
                            case DialogTypes.CustomerAddressType.Sms: return Qt.ImhDigitsOnly | Qt.ImhDialableCharactersOnly;
                            case DialogTypes.CustomerAddressType.Email: return Qt.ImhNoAutoUppercase | Qt.ImhEmailCharactersOnly;
                            }
                        }

                        RegularExpressionValidator {
                            id: emailValidator
                            regularExpression: /^[a-zA-Z0-9_.+-]+@[a-zA-Zа-яА-ЯЁё0-9-]+\.[a-zA-Zа-яА-ЯЁё0-9-.]{2,}$/
                        }

                        validator: {
                            switch (selectedType) {
                            case DialogTypes.CustomerAddressType.Email: return emailValidator
                            default: return null
                            }
                        }
                        inputMask: {
                            switch (selectedType) {
                            case DialogTypes.CustomerAddressType.Sms: return "+7(999)999-99-99"
                            default: return "";
                            }
                        }

                        Component.onCompleted: () => {
                            this.text = root.action.defaultAddress
                            this.forceActiveFocus()
                        }
                    }
                }

                Space { objectName: "fillSpacer"; Layout.fillHeight: true }
            }
        }

        RowLayout {
            objectName: "buttonsLayout"
            spacing: 40

            ActionButton {
                objectName: "cancelButton"
                Style.class: "customerAddressCancel"
                implicitWidth: 1

                Tr { tr: root.action.cancelButton; fallback: "addressCancel" }

                onClicked: () => root.reject()

                Layout.fillWidth: true
                Layout.preferredHeight: 110
            }

            ActionButton {
                objectName: "applyButton"
                Style.class: "customerAddressApply"
                implicitWidth: 1
                enabled: inputField.acceptableInput

                Tr { tr: root.action.okButton; fallback: "addressAccept" }

                onClicked: () => root.accept()

                Layout.fillWidth: true
                Layout.preferredHeight: 110
            }
        }
    }
}
