import QtQml.Base
import QtQuick
import QtQuick.Layouts

import Core
import I18n
import Sco
import Dialog

import "../core"
import "../widgets"

ModalForm {
    id: root
    Style.class: "inputForm"

    property var action: DialogState.actions[Context.id.value]

    onActivated: () => {
        controls.inputField.forceActiveFocus()
        if (controls.inputField.text && !(controls.inputField.inputMask.length > 0))
            controls.inputField.selectAll()
    }

    Context.context {
        inputSources: InputSources.inputSources(action.inputSources)
        muteScanner: action.checkScannerRegexp && Config.getBool("Dialog:checkInputRegexp", true)
        logoActionInfo: (action.hideInput || hideMarkInputCancel) ? (() => this.logoAction()) : false
    }

    RegularExpressionValidator { id: regexpValidator; regularExpression: new RegExp(root.action.regexp) }

    readonly property bool kbdInput: (Context.inputSources & InputSources.Keyboard) != 0
    readonly property bool showCancel: (action.showCancel && (!root.hideMarkInputCancel || action.textType != DialogTypes.TextType.Mark || !Sco.customerMode))

    property bool hideMarkInputCancel: false
    property bool callAttendantOnHideMarkInputCancel: false
    property bool showControls: false

    Component.onCompleted: () => {
        if (action.textType == DialogTypes.TextType.Mark && Sco.customerMode && !action.hideControls) {
            this.hideMarkInputCancel = Config.getBool("Dialog:hideMarkInputCancel")
            this.callAttendantOnHideMarkInputCancel = Config.getBool("Dialog:callAttendantOnMarkInputCancel")
        }
    }

    Keys.onEnterPressed: () => root.accept()
    Keys.onReturnPressed: () => root.accept()
    Keys.onEscapePressed: () => root.reject()

    function logoAction() {
        if (hideMarkInputCancel && !cancelButton.visible) {
            const action = Action.create("AUTH_CALLATTENDANT", { showCancel: true, $parent: this.action })
            PluginManager.async(action)
        } else {
            root.showControls = true
            image.visible = image.visible && !action.hideControls
        }
    }

    function accept() {
        if (!action.canBeEmpty && (!controls.inputField.acceptableInput || controls.inputField.text.length === 0))
            return
        const setText = !(action.textType === DialogTypes.TextType.Mark && action.canBeEmpty)
        action.text = setText ? controls.inputField.text : ""
        action.canceled = false
        removeContext()
    }

    function reject() {
        if (callAttendantOnHideMarkInputCancel && Sco.customerMode) {
            PluginManager.async("AUTH_CALLATTENDANT", { $parent: this.action })
            return
        }
        action.text = String()
        action.canceled = true
        removeContext()
    }

    contentItem: ColumnLayout {
        objectName: "mainLayout"
        spacing: 40

        SLabel {
            id: title
            objectName: "title"
            Style.class: "inputFormTitle"
            horizontalAlignment: SLabel.AlignHCenter
            wrapMode: SLabel.WordWrap

            Tr { tr: action.title }

            Layout.fillWidth: true
        }

        SLabel {
            id: message
            objectName: "message"
            Style.class: "inputFormMessage"
            horizontalAlignment: SLabel.AlignHCenter
            wrapMode: SLabel.WordWrap
            visible: text.length > 0

            Tr {
                tr: controls.visible || action.hideControlsMsg.empty
                        ? action.message
                        : action.hideControlsMsg
            }

            Layout.fillWidth: true
        }

        SvgImage {
            id: image
            objectName: "image"
            visible: !kbdInput.visible

            property bool forceBarcode: action.image.type == ImageData.None && action.textType == DialogTypes.TextType.Barcode

            source: forceBarcode ? "hint.barcode" : action.image

            Layout.preferredHeight: forceBarcode ? Style.parameter("geometry.inputBarcodeImageHeight") : action.imageHeight != -1 ? action.imageHeight : implicitHeight
            Layout.alignment: Qt.AlignHCenter
        }

        InputControls {
            id: controls
            objectName: "inputControls"
            visible: (!root.action.hideControls || root.showControls) && (root.kbdInput || root.showCancel)

            useDigits: true
            okButtonTr: action.okButton
            cancelButtonTr: action.cancelButton

            kbdInput.visible: root.kbdInput
            inputField {
                id: inputField
                objectName: "inputField"
                echoMode: root.action.hideInput ? TextInput.Password : TextInput.Normal
                validator: root.action.regexp.length > 0 ? regexpValidator : null
                inputMask: root.action.mask
                horizontalAlignment: Qt.AlignHCenter

                Component.onCompleted: () => {
                    if (root.action.text.length > 0) {
                        inputField.text = root.action.text
                        inputField.selectAll()
                    }
                }
            }

            buttonsLayout.visible: root.kbdInput || root.showCancel
            okButton {
                objectName: "okButton"
                visible: root.kbdInput || (root.action.canBeEmpty && root.showCancel)
                enabled: root.action.canBeEmpty || controls.inputField.acceptableInput && controls.inputField.text.length > 0

                onClicked: () => root.accept()
            }
            cancelButton {
                id: cancelButton
                objectName: "cancelButton"
                visible: root.showCancel

                onClicked: () => root.reject()
            }
            Layout.alignment: Qt.AlignHCenter
        }

        ColumnLayout {
            id: additionalButtons

            property bool showManualBarcode: !controls.visible && action.textType == DialogTypes.TextType.Barcode
            property bool showHelp: !controls.visible && !showManualBarcode
            property bool show: showManualBarcode || showHelp
            visible: show
            Layout.fillWidth: true

            ActionButton {
                id: manualBarcode
                objectName: "manualInputButton"
                Style.class: "inputBarcodeManual"
                icon: Style.parameter("icon.input.manual")

                visible: additionalButtons.showManualBarcode

                Tr { code: "checkManualInputBarcodeManual" }

                onClicked: () => root.showControls = true

                Layout.fillWidth: true
                Layout.preferredHeight: 105
            }

            HelpButton {
                id: help
                objectName: "helpButton"
                Style.class: "inputHelpButton"
                visible: parent.showHelp

                Tr { code: "inputHelpButton" }

                Layout.fillWidth: true
                Layout.preferredHeight: 105
            }
        }
    }
}
