import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import Core
import Dialog
import I18n

import "../widgets"
import "../core"

ModalForm {
    id: root
    Style.class: "inputQuantityForm"
    Context.inputSources: InputSources.Keyboard

    property Component simpleInfo: SLabel {
        Style.class: "inputQuantityMessage"
        wrapMode: SLabel.WordWrap
        Tr { tr: action.message }
        horizontalAlignment: Text.AlignHCenter
    }

    property Component extendedInfo: GoodsCard {
        name: action.name
        barcode: action.barcode
        image: action.image.exists ? action.image : (action.showImagePlaceholder ? Style.parameter("image.inputQuantity.placeholder") : "")
        price: action.price

        labelName: "inputQuantityName"
        labelBarcode: "inputQuantityBarcode"
        moneyPrice: "inputQuantityPrice"
    }

    property var action: DialogState.actions[Context.id.value]
    property var minQuantity: parseStr(action.minQuantity.str)

    function parseStr(value) { return action.fractional ? parseFloat(value) : parseInt(value) }

    Keys.onEnterPressed: () => okButton.clicked()
    Keys.onReturnPressed: () => okButton.clicked()
    Keys.onEscapePressed: () => cancelButton.clicked()

    ColumnLayout {
        id: layout
        objectName: "mainLayout"
        anchors.left: parent.left
        anchors.right: parent.right
        spacing: 20

        SLabel {
            id: title
            Layout.fillWidth: true
            wrapMode: SLabel.WordWrap
            Style.class: "inputQuantityTitle"
            Tr { tr: action.title }
            horizontalAlignment: Text.AlignHCenter
        }
        Loader {
            objectName: "goodsItemLoader"
            Layout.topMargin: 40
            Layout.fillWidth: true
            sourceComponent: action.extendedInfo ? extendedInfo : simpleInfo
        }
        RowLayout {
            spacing: 10
            Item {
                objectName: "minusButton"
                Layout.fillWidth: true
                Layout.preferredHeight: 80
                property bool iconAntialiasing: false
                property int iconSize: Style.parameter("geometry.actionButton.iconSize", 32)
                ActionButton {
                    id: minus
                    Style.class: "inputQuantityMinusButton"
                    anchors.fill: parent
                    iconSize: parent.iconSize
                    iconAntialiasing: parent.iconAntialiasing
                    enabled: !root.action.fractional
                    icon: enabled ? Style.parameter("icon.inputQuantity.minus")
                                  : Style.parameter("icon.inputQuantity.minus_disable")
                    onClicked: input.text = parseStr(input.text) - 1
                }
            }
            Item {
                Layout.fillWidth: true
                Layout.preferredHeight: 80
                objectName: "inputField"
                SGroupBox {
                    anchors.fill: parent
                    Style.class: "inputQuantityInputField"
                    FocusScope {
                        anchors.fill: parent
                        focus: true
                        STextEdit {
                            id: input
                            anchors.centerIn: parent
                            Style.class: "inputQuantityInputField"
                            text: Math.max(parseStr(action.quantity.str), minQuantity)
                            focus: true
                            IntValidator { id: iv; bottom: minQuantity; top: 999999 }
                            RegularExpressionValidator { id: dv; regularExpression: new RegExp(/^([1-9]\d{0,5}|0[1-9]{0,1})(\.\d{0,3})/) }
                            validator: action.fractional ? dv : iv
                            onTextChanged: () => {
                                let text = this.text.replace(/^0*/, "")
                                let parsedValue = parseStr(text)
                                if (!action.fractional && parsedValue >= iv.top) {
                                    plus.enabled = false
                                    return
                                }
                                plus.enabled = !root.action.fractional
                                const isMin = (parsedValue > minQuantity)
                                minus.enabled = isMin && !root.action.fractional
                                okButton.enabled = isMin
                                if (text.length === 0 || text[0] === '.')
                                    text = "0" + text
                                this.text = text
                            }
                            onFocusChanged: () => { if (focus) input.selectAll() }
                            Component.onCompleted: this.selectAll()
                        }
                    }
                }
            }
            Item {
                Layout.fillWidth: true
                Layout.preferredHeight: 80
                objectName: "plusButton"
                property bool iconAntialiasing: false
                property int iconSize: Style.parameter("geometry.actionButton.iconSize", 32)
                ActionButton {
                    id: plus
                    Style.class: "inputQuantityPlusButton"
                    anchors.fill: parent
                    enabled: !root.action.fractional
                    iconSize: parent.iconSize
                    iconAntialiasing: parent.iconAntialiasing
                    icon: enabled ? Style.parameter("icon.inputQuantity.plus")
                                  : Style.parameter("icon.inputQuantity.plus_disabled")
                    onClicked: input.text = (input.text.length > 0 ? parseStr(input.text) : 0) + 1
                }
            }
        }
        InlineKeyboard {
            objectName: "inlineKeyboard"
            Layout.preferredHeight: 350
            Layout.fillWidth: true
            layoutType: "digits"
        }
        GridLayout {
            objectName: "controlButtonsLayout"
            Layout.fillWidth: true
            property bool vertical: true
            flow: vertical ? GridLayout.TopToBottom : GridLayout.LeftToRight
            layoutDirection: vertical ? Qt.LeftToRight : Qt.RightToLeft
            rowSpacing: 15
            columnSpacing: 15

            ActionButton {
                id: okButton
                objectName: "okButton"
                Layout.preferredHeight: 110
                Layout.fillWidth: true
                implicitWidth: 1
                Style.class: "inputOkButton"
                Tr { tr: action.okButton; fallback: "inputQuantityOk" }
                onClicked: () => {
                    if (!enabled)
                        return
                    if (input.text.length == 0) {
                        root.action.quantity = Quantity.zero
                    } else if (root.action.fractional) {
                        root.action.quantity = Quantity.fromGrams(input.text * 1000)
                    } else {
                        root.action.quantity = Quantity.fromCount(input.text)
                    }
                    root.action.inputCanceled = false
                    removeContext()
                }
            }
            ActionButton {
                id: cancelButton
                objectName: "cancelButton"
                Layout.preferredHeight: 110
                Layout.fillWidth: true
                implicitWidth: 1
                visible: root.action.showCancel
                Style.class: "inputCancelButton"
                Tr { tr: action.cancelButton; fallback: "inputQuantityCancel" }
                icon: Style.parameter("icon.inputQuantity.cancel")
                onClicked: () => {
                    root.action.quantity = Quantity.zero
                    root.action.inputCanceled = true
                    removeContext()
                }
            }
        }
    }
}
