import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQml.Models
import Qt5Compat.GraphicalEffects

import Check
import Core
import I18n
import PickList as PL

import "../core"
import "../widgets"

Item {
    id: root

    property size grid: Qt.size(4, 2)
    property real cellWidth: treeView.currentItem.width / grid.width
    property real cellHeight: treeView.currentItem.height / grid.height
    property bool isDialogPickList: PL.PickList.dialogDataIndex.valid
    property var model: PL.PickList.pageModel
    property alias title: title
    property alias navigationBox: navigationBox
    property Logger log: Utils.createLogger("PickList")

    signal selected(index: var)
    signal hideKeyboard()
    signal clearFilter()

    function logSelectedItem(index: var) {
        let item = model.modelItemData(index)
        if (!item)
            return
        log?.info("Выбран элемент в каталоге товаров", [
            {name: "Название", value: item.text},
            {name: "Группа", value: item.isGroup},
            {name: "Id", value: item.id},
        ])
    }

    Connections {
        target: PL.PickList

        function onShowCodes() {
            let index = PL.PickList.pageModel.codesIndex()
            if (index.valid) {
                while (treeView.depth > 1)
                    treeView.pop()
                treeView.push(index)
                title.text = title.regenerateTitle()
            }
        }
    }

    component PickListItem: SGroupBox {
        id: itemBox
        Style.class: "pickListItem"

        leftInset: 5
        topInset: 5
        rightInset: 5
        bottomInset: 5
        padding: Style.parameter("geometry.pickList.itemDelegate.padding", 30)

        contentItem: Item {
            SLabel {
                anchors {
                    left: parent.left
                    top: parent.top
                    right: parent.right
                }

                Style.class: "pickListItem"
                text: model.name
                wrapMode: SLabel.WordWrap
                elide: SLabel.ElideRight
                maximumLineCount: 2
            }

            SLabel {
                anchors {
                    left: parent.left
                    bottom: parent.bottom
                }

                visible: !model.isGroup && !model.price.zero
                Style.class: "pickListItemPrice"
                text: model.price.str + (PL.PickList.showUnits && model.unit.length > 0 ? "\n/" + model.unit : "")
            }
        }

        background: Control {
            id: itemBoxBackground

            background: Rectangle {
                anchors.fill: parent
                color: itemBox.Style.background.color
                radius: itemBox.Style.border.radius
            }

            contentItem: Rectangle {
                color: "transparent"
                radius: itemBox.Style.border.radius

                border {
                    width: itemBox.Style.border.width
                    color: itemBox.Style.border.color
                }

                Behavior on color {
                    ColorAnimation { duration: Style.animationDuration("pickList.item", 100) }
                }

                // Маска, чтобы изображение элемента визуально не вылезало за границы
                layer {
                    enabled: true
                    effect: OpacityMask {
                        maskSource: Rectangle {
                            anchors.centerIn: parent
                            width: itemBoxBackground.width
                            height: itemBoxBackground.height
                            radius: itemBox.Style.border.radius
                        }
                    }
                }

                // Задаем z=-1, чтобы изображение не перекрывало "границы" background'а
                SvgImage {
                    z: -1
                    anchors {
                        top: parent.top; left: parent.left
                        topMargin: Style.parameter("geometry.pickList.itemDelegate.imageTopMargin", 70)
                        leftMargin: Style.parameter("geometry.pickList.itemDelegate.imageLeftMargin", 90)
                    }

                    height: 140 * Style.parameter("geometry.pickList.itemDelegate.imageScale", 1)
                    width: 170 * Style.parameter("geometry.pickList.itemDelegate.imageScale", 1)

                    source: model.image
                }
            }
        }
    }

    property Component searchItem: RowLayout {
        anchors.fill: parent
        spacing: 15

        SGroupBox {
            id: pickListSearch
            objectName: "pickListSearch"
            Style.class: "pickListSearch"
            Layout.fillWidth: true

            property bool showSearch: search.text.length > 0 || search.activeFocus

            contentItem: STextEdit {
                id: search
                clip: true
                horizontalAlignment: STextEdit.AlignHCenter
                verticalAlignment: STextEdit.AlignVCenter
                Style.class: "pickListSearch"
                Layout.fillWidth: true
                focus: Qt.inputMethod.visible

                property var filterIndex: root.model.filterIndex(displayText)

                Connections {
                    target: treeView

                    function onBeforePop(oldIndex) {
                        if (oldIndex === search.filterIndex)
                            Qt.callLater(() => root.clearFilter())
                    }

                    function onBeforePush(oldIndex) {
                        if (oldIndex === search.filterIndex)
                            Qt.callLater(() => root.clearFilter())
                    }
                }

                Connections {
                    target: root

                    function onClearFilter() {
                        search.clear()
                    }
                }

                Timer {
                    id: filterTimer
                    repeat: false
                    interval: 500

                    onTriggered: () => {
                        let trimmed = search.displayText.trim()
                        if (trimmed.length > 0) {
                            root.log?.info("Поиск в каталоге товаров", [{name: "Строка поиска", value: trimmed}])
                            search.filterIndex = root.model.filterIndex(trimmed, treeView.currentItem.index)
                            treeView.push(search.filterIndex)
                        } else {
                            root.clearFilter()
                        }
                    }
                }

                onDisplayTextChanged: () => {
                    if (search.displayText.length > 0) {
                        filterTimer.restart()
                    } else {
                        filterTimer.stop()
                        if (treeView.currentItem.index === search.filterIndex)
                            treeView.pop()
                    }
                }
            }
            ActionButton {
                Style.class: "pickListSearch"
                objectName: "pickListSearchButton"
                icon: Style.parameter("icon.pickList.search")
                Tr { code: "pickListSearchHint" }
                anchors.fill: parent
                visible: !parent.showSearch
            }

            MouseArea {
                anchors.fill: parent
                preventStealing: true
                enabled: !parent.showSearch

                onClicked: () => search.forceActiveFocus(Qt.MouseFocusReason)
            }
        }

        ActionButton {
            objectName: "pickListDetectButton"
            Style.class: "pickListDetect"
            Tr { code: "pickListDetectItem" }
            visible: !root.isDialogPickList && PluginManager.hasHandler(action)
            action: "GOODSDETECTOR_DETECT"
            Layout.preferredWidth: parent.width / 3
            Layout.fillHeight: true
        }
    }

    ColumnLayout {
        id: navigationBox
        anchors.fill: parent
        spacing: 10

        SLabel {
            id: title
            horizontalAlignment: Qt.AlignHCenter
            verticalAlignment: Qt.AlignVCenter
            Style.class: "pickListTitle"
            text: regenerateTitle()
            elide: SLabel.ElideLeft

            Layout.alignment: Qt.AlignHCenter | Qt.AlignTop
            Layout.bottomMargin: 20
            Layout.fillWidth: true

            function regenerateTitle() {
                const titles = []
                const rootIndex = root.model.rootIndex()
                let index = treeView.currentItem.index

                while (index !== rootIndex) {
                    const name = root.model.data(index, PL.PageModel.Name)
                    if (name.length > 0 && I18n.ui(name).length > 0)
                        titles.push(I18n.ui(name));
                    index = root.model.parent(index)
                }
                let rootTitle = root.model.data(PL.PickList.dialogDataIndex, PL.PageModel.Name)
                titles.push(rootTitle.length > 0 ? rootTitle : I18n.ui('pickListLayerRoot'))
                titles.reverse()
                return titles.join(' / ')
            }

            Connections {
                target: root.model

                function onDataChanged() {
                    title.text = title.regenerateTitle()
                }
            }

            Connections {
                target: treeView

                function onAfterPush() {
                    title.text = title.regenerateTitle()
                }

                function onAfterPop() {
                    title.text = title.regenerateTitle()
                }
            }

            Connections {
                target: I18n

                function onCurrentLangChanged() {
                    title.text = title.regenerateTitle()
                }
            }
        }

        Loader {
            objectName: "searchLoader"
            property real searchHeight: 90

            sourceComponent: root.isDialogPickList ? null : root.searchItem
            Layout.fillWidth: true
            Layout.preferredHeight: root.isDialogPickList ? 0 : searchHeight
        }

        TreeView {
            id: treeView
            model: root.model
            clip: true

            Connections {
                target: root.model

                function onModelReset() {
                    while (treeView.depth > 1)
                        treeView.pop()
                }
            }

            Transition {
                id: fadeIn

                NumberAnimation {
                    property: "opacity"; from: 0; to: 0
                    duration: Style.animationDuration("transition.pickList", 250)
                }

                NumberAnimation {
                    property: "opacity"; from: 0; to: 1
                    duration: Style.animationDuration("transition.pickList", 250)
                }
            }

            Transition {
                id: fadeOut

                NumberAnimation {
                    property: "opacity"; from: 1; to: 0
                    duration: Style.animationDuration("transition.pickList", 250)
                }

                NumberAnimation {
                    property: "opacity"; from: 0; to: 0
                    duration: Style.animationDuration("transition.pickList", 250)
                }
            }

            pushEnter: fadeIn
            pushExit: fadeOut
            popEnter: fadeIn
            popExit: fadeOut
            replaceEnter: fadeIn
            replaceExit: fadeOut

            delegate: Item {
                id: listItem

                property var model
                property var index
                property alias count: view.count
                property alias currentIndex: view.currentIndex

                SwipeView {
                    focusPolicy: Qt.ClickFocus

                    id: view
                    anchors.fill: parent

                    Repeater {
                        model: PagedModel {
                            pageSize: root.grid.width * root.grid.height
                            source: listItem.model
                            rootIndex: listItem.index

                            Component.onCompleted: {
                                if (root.isDialogPickList) {
                                    rootIndex = PL.PickList.dialogDataIndex
                                    title.regenerateTitle()
                                }
                            }
                        }

                        delegate: Control {
                            background: MouseArea {
                                anchors.fill: parent
                                onClicked: () => {
                                    treeView.forceActiveFocus(Qt.MouseFocusReason)
                                    root.hideKeyboard()
                                }
                            }

                            GridView {
                                anchors.fill: parent
                                id: gridView
                                model: page
                                interactive: false

                                cellWidth: root.cellWidth
                                cellHeight: root.cellHeight

                                add: fadeIn
                                remove: fadeOut

                                delegate: PickListItem {
                                    width: gridView.cellWidth
                                    height: gridView.cellHeight

                                    focusPolicy: Qt.ClickFocus

                                    onClicked: () => {
                                        let selected = page.modelIndex(index)
                                        if (model.isGroup) {
                                            if (treeView.currentItem.index === root.model.filterIndex()) {
                                                selected = root.model.processAfterFilter(selected)
                                                while (treeView.depth > 1)
                                                    treeView.pop()
                                            }
                                            treeView.push(selected)
                                        } else
                                            root.selected(selected)
                                        logSelectedItem(selected)
                                    }
                                }
                            }
                        }
                    }
                }
            }

            Layout.fillWidth: true
            Layout.fillHeight: true
        }

        RowLayout {
            ActionButton {
                Style.class: "pickListBack"
                icon: !enabled ? Style.parameter("icon.pickList.backDisabled") : Style.parameter("icon.pickList.backActive")
                iconSize: 24
                enabled: treeView.depth > 1

                onClicked: () => treeView.pop()

                Layout.preferredWidth: 100
                Layout.preferredHeight: 56
            }

            Space { Layout.fillWidth: true }

            PageIndicator {
                id: pageIndicator
                opacity: count > 1 ? 1 : 0
                count: treeView.currentItem.count
                currentIndex: treeView.currentItem.currentIndex
                spacing: 10

                delegate: Rectangle {
                    Style.type: "PageIndicatorItem"
                    Style.class: "pickList"
                    width: 16
                    height: 16
                    property bool checked: index === pageIndicator.currentIndex
                    color: Style.background.color
                    radius: Style.border.radius
                    border.width: Style.border.width
                    border.color: Style.border.color

                    Behavior on color { ColorAnimation { duration: Style.animationDuration("pickList.pageIndicator", 100) } }
                    Behavior on border.color { ColorAnimation { duration: Style.animationDuration("pickList.pageIndicator", 100) } }
                }

                Behavior on opacity { NumberAnimation { duration: Style.animationDuration("pickList.pageIndicator", 250) } }

                Layout.preferredWidth: implicitWidth
                Layout.preferredHeight: implicitHeight
                Layout.alignment: Qt.AlignHCenter
            }

            Space { Layout.fillWidth: true }

            ActionButton {
                Style.class: "pickListLeft"
                icon: !enabled ? Style.parameter("icon.pickList.leftDisabled") : Style.parameter("icon.pickList.leftActive")
                iconSize: 24
                enabled: treeView.currentItem.currentIndex > 0

                onClicked: () => --treeView.currentItem.currentIndex

                Layout.preferredWidth: 56
                Layout.preferredHeight: 56
            }

            ActionButton {
                Style.class: "pickListRight"
                icon: !enabled ? Style.parameter("icon.pickList.rightDisabled") : Style.parameter("icon.pickList.rightActive")
                iconSize: 24
                enabled: treeView.currentItem.currentIndex < (treeView.currentItem.count - 1)

                onClicked: () => ++treeView.currentItem.currentIndex

                Layout.preferredWidth: 56
                Layout.preferredHeight: 56
            }
        }
    }
}
