import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import Check
import Core
import I18n

Control {
    id: root

    Layout.fillWidth: true
    Layout.fillHeight: true

    property int currentIndex: -1
    property var currentMethod: model.get(currentIndex)

    property alias title: title
    property alias message: message
    property alias hint: hint
    property alias controls: controls
    property alias okButtonConnections: okButtonConnections
    property alias cancelButtonConnections: cancelButtonConnections

    property string inputMethodClass: "inputMethod"
    property var titleTr: null
    property var messageTr: null

    property ListModel methods: ListModel { id: model }
    property var append: function(method) { model.append(method) }

    Component.onCompleted: () => {
        currentIndex = 0
    }

    property var accept: function() {
        if (!controls.inputField.acceptableInput)
            return

        removeContext()
    }

    property var reject: function() {
        removeContext()
    }

    Connections {
        id: okButtonConnections
        target: controls.okButton

        function onClicked() {
            accept()
        }
    }

    Connections {
        id: cancelButtonConnections
        target: controls.cancelButton

        function onClicked() {
            reject()
        }
    }

    Keys.onEnterPressed: () => accept()
    Keys.onReturnPressed: () => accept()
    Keys.onEscapePressed: () => reject()

    onCurrentMethodChanged: () => {
        controls.inputField.text = ''
        if (currentMethod)
            Check.cardInputMethod = currentMethod.cardInputMethod
    }

    contentItem: ColumnLayout {
        objectName: "inputMethodMainLayout"
        spacing: 40
        anchors.fill: parent

        SLabel {
            id: title

            Style.class: "black50px700"
            horizontalAlignment: SLabel.AlignHCenter
            wrapMode: SLabel.WordWrap

            Tr { tr: root.titleTr; fallback: "inputMethodListTitle" }

            Layout.fillWidth: true
        }

        SLabel {
            id: message

            Style.class: "grayDark24px600"
            horizontalAlignment: SLabel.AlignHCenter
            wrapMode: SLabel.WordWrap
            visible: model.count > 1

            Tr { tr: root.messageTr; fallback: "inputMethodListMessage" }

            Layout.fillWidth: true
        }

        Item {
            id: buttonsLayout
            objectName: "inputMethodButtonsLayout"
            property real spacing: 10
            property bool modelOdd: (model.count % 2) != 0
            property int rows: (model.count + modelOdd) / 2
            property int buttonsHeight: 71

            visible: model.count > 1
            Layout.fillWidth: true
            Layout.preferredHeight: rows * buttonsHeight + (rows - 1) * spacing

            Repeater {
                model: model
                delegate: ActionButton {
                    Style.class: root.inputMethodClass

                    property bool odd: (model.index % 2) != 0
                    property int row: ((model.index + buttonsLayout.modelOdd) / 2)
                    property int col: buttonsLayout.modelOdd ? ((model.index + !!model.index) % 2) : (model.index % 2)

                    width: buttonsLayout.modelOdd && model.index == 0 ? buttonsLayout.width : (buttonsLayout.width - buttonsLayout.spacing) / 2
                    height: buttonsLayout.buttonsHeight
                    x: col == 0 ? 0 : (width + buttonsLayout.spacing)
                    y: row * (height + buttonsLayout.spacing)
                    enabled: root.currentIndex != model.index
                    Tr { code: model.button }

                    onClicked: () => currentIndex = model.index
                }
            }
        }

        SLabel {
            id: hint
            objectName: "hintLabel"

            Style.class: "grayDark30px600"
            horizontalAlignment: Qt.AlignHCenter
            wrapMode: SLabel.WordWrap
            visible: currentMethod.hint !== void 0 && currentMethod.hint.length > 0

            Tr { code: currentMethod.hint ? currentMethod.hint : "" }

            Layout.fillWidth: true
        }

        SvgImage {
            objectName: "inputMethodImage"
            visible: !currentMethod.manual && currentMethod.hintStyle !== void 0
            source: Style.parameter(currentMethod.hintStyle)

            Layout.fillWidth: true
            Layout.leftMargin: 90
            Layout.topMargin: 80
            Layout.rightMargin: 90
            Layout.bottomMargin: 46
            Layout.alignment: Qt.AlignHCenter
        }

        DoubleValidator { id: dv }

        InputControls {
            id: controls
            objectName: "inputControls"

            kbdInput.visible: currentMethod.manual
            inputField {
                objectName: "inputField"
                echoMode: TextInput.Normal

                validator: (new Set([Check.CardInputMethod.Number, Check.CardInputMethod.PaymentCode])).has(currentMethod.cardInputMethod) ? dv : null
                inputMask: currentMethod.cardInputMethod == Check.CardInputMethod.Phone ? "+7(999)999-99-99" : ""

                horizontalAlignment: Qt.AlignHCenter
            }

            okButton {
                objectName: "okButton"
                Style.class: "ActionButton"

                visible: currentMethod.manual
                enabled: controls.inputField.acceptableInput
            }
            cancelButton {
                objectName: "cancelButton"
                Style.class: "ActionButton"
            }
        }
    }
}
