import QtQuick
import QtQml.Models
import QtQuick.Controls
import QtQuick.Layouts

import I18n
import Core

/**
 * @brief Кнопка выбора языка
 *
 * @param model модель доступных языков
 */
Control {
    id: root

    property Popup popup
    property real itemWidth: 194
    property real itemSpacing: 20
    property LanguageModel model
    property color backgroundColor

    enabled: root.model.count > 1

    component LanguageButton: ActionButton {
        id: langButton
        Style.class: "lang"
        property string lang
        icon: Style.parameter("icon.lang." + langButton.lang)
        iconSize: 32
        Tr { code: "langName." + langButton.lang }
        height: root.height
        width: itemWidth
        background: Rectangle {
            color: root.backgroundColor
            radius: langButton.Style.border.radius
            border.color: langButton.Style.border.color
            border.width: langButton.Style.border.width
        }
    }


    LanguageButton {
        id: currentLangButton
        lang: I18n.currentLang
        visible: !popup.opened
        onClicked: () => {
            if (root.model.count === 2) {
                const first = root.model.at(0).lang
                if (I18n.currentLang === first)
                    I18n.currentLang = root.model.at(1).lang
                else
                    I18n.currentLang = first
            } else if (root.model.count > 2) {
                popup.open()
            } else {
                // unrechable
            }
        }
    }

    popup: Popup {
        id: popup
        x: -(width - itemWidth)
        width: list.implicitWidth
        height: root.height
        padding: 0

        Timer {
            id: timer
            interval: 5000
            repeat: false

            onTriggered: () => popup.close()
        }

        onOpened: () => timer.start()

        onClosed: () => timer.stop()

        enter: Transition {
            NumberAnimation {
                easing.type: Easing.InOutQuad;
                property: "width"; from: itemWidth; to: list.fullWidth
                duration: Style.animationDuration("language", 500)
            }

            NumberAnimation {
                easing.type: Easing.InOutQuad;
                property: "x"; from: 0; to: -(list.fullWidth - itemWidth)
                duration: Style.animationDuration("language", 500)
            }
        }

        exit: Transition {
            NumberAnimation {
                easing.type: Easing.InOutQuad;
                property: "width"; to: itemWidth
                duration: Style.animationDuration("language", 500)
            }

            NumberAnimation {
                easing.type: Easing.InOutQuad;
                property: "x"; to: 0
                duration: Style.animationDuration("language", 500)
            }
        }

        PathView {
            id: list
            anchors.fill: parent
            property real fullWidth: itemWidth * model.count
            interactive: false

            model: root.model

            delegate: LanguageButton {
                z: I18n.currentLang === model.lang ? root.model.count : root.model.count - 1 - model.row
                lang: model.lang
                width: itemWidth

                onClicked: () => {
                    popup.close()
                    I18n.currentLang = lang
                }
            }

            /**
                * Элементы рисуются на линии таким образом:
                *
                * Линия:
                *  *-----*-----*-----
                *
                * Элемент:
                *  +---------+
                *  |    *    |
                *  +---------+
                *
                * Каждый элемент привязывается звездочкой к соотвестсвующей звездочке на линии
                */
            path: Path {
                startX: itemWidth / 2 - 1; startY: root.height / 2

                PathLine {
                    x: list.width - (itemWidth / 2) + itemWidth * ((list.width - itemWidth) / (list.fullWidth - itemWidth))
                    y: root.height / 2
                }
            }
        }

        background: Rectangle {
            color: root.backgroundColor
            radius: currentLangButton.Style.border.radius
        }
    }
}
